package ro.sync.util.editorvars.parser.model;

import java.util.List;

/**
 * A parameter of an editor variable. It can contain a list of 
 * text parts and editor variables.
 */
public class CompoundLexicalItem extends LexicalItem {
  
  /**
   * The lexical units that form the parameter.
   */
  private List<LexicalItem> lexicalItems;
  
  /**
   * Constructor.
   * 
   * @param lexicalItems The lexical units that form the parameter.
   */
  public CompoundLexicalItem(List<LexicalItem> lexicalItems) {
    super(Type.COMPOUND_ITEM);
    this.lexicalItems = lexicalItems;
  }

  /**
   * @return The lexical units that form the parameter.
   */
  public List<LexicalItem> getLexicalItems() {
    return lexicalItems;
  }

  @Override
  public String toString() {
    return "'" + lexicalItems.toString() + "', type: " + getType();
  }
  
  @Override
  public String getStringValue() {
    StringBuilder sb = new StringBuilder();
    int size = lexicalItems.size();
    for (int i = 0; i < size; i++) {
      LexicalItem item = lexicalItems.get(i);
      sb.append(item.getStringValue());
    }
    
    return sb.toString();
  }
  
}
