package ro.sync.util.editorvars.parser.model;

import java.util.Collections;
import java.util.List;

import ro.sync.util.editorvars.expander.EditorVariableUtil;

/**
 * An editor variable.
 */
public class EditorVariable extends LexicalItem {
  /**
   * The name of the editor variable (ask, answer, cfn, pd, etc).
   */
  private String name;
  
  /**
   * The variable's parameters.
   */
  private List<LexicalItem> params;

  /**
   * Constructor.
   * 
   * @param name    Variable name (ask, answer, cfn, pd, etc).
   * @param params  The variable's parameters.
   */
  public EditorVariable(String name, List<LexicalItem> params) {
    super(Type.EDITOR_VARIABLE);
    this.name = name;
    
    setParams(params);
  }
  
  /**
   * Constructor.
   * 
   * @param name Variable name.
   * @param type Variable type.
   */
  protected EditorVariable(String name, Type type) {
    super(type);
    this.name = name;
  }
  
  /**
   * @return Variable name.
   */
  public String getName() {
    return name;
  }

  /**
   * @return The parameters or an empty list. Never <code>null</code>.
   */
  public List<LexicalItem> getParams() {
    return params;
  }

  /**
   * Sets the variable parameters.
   * 
   * @param params Variable parameters.
   */
  public void setParams(List<LexicalItem> params) {
    this.params = params != null ? params : Collections.emptyList();
  }
  
  /**
   * @return A string representation of the parameters.
   */
  private String paramsToString() {
    StringBuilder stringBuilder = new StringBuilder();
    stringBuilder.append("(");
    int size = params.size();
    for (int i = 0 ; i < size; i++) {
      LexicalItem param = params.get(i);
      stringBuilder.append("\n    ").append(param.toString());
    }
    stringBuilder.append("\n  )");
    return stringBuilder.toString();
  }
  
  /**
   * @return A string representation of the editor variable.
   */
  @Override
  public String toString() {
    return "'${" + name + (params != null && !params.isEmpty() ? paramsToString() + "\n" : "") + "}'"
        + ", type: " + getType();
  }
  
  @Override
  public String getStringValue() {
    return EditorVariableUtil.serialize(this);
  }
}
