<?xml version="1.0" encoding="UTF-8"?>
<!-- 
    Generates different FO elements based on the css:display attribute:
    
    css:display="block" -> fo:block
    css:display="inline" -> fo:inline
    etc..

	The @css:link attribute is examined to generate a wrapping element around the fo:inline and fo:block.
        
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:oxy="http://www.oxygenxml.com/css2fo" xmlns:css="http://www.w3.org/1998/CSS"
    exclude-result-prefixes="xs oxy css" version="2.0">

    <!-- Fast search for IDs. -->
    <xsl:key name="ids" match="*" use="@id|@xml:id"/>
    <xsl:key name="html-names" match="*:html//*:a" use="@name"/>


    <!-- 
    
    We can have links only in inline context, on text, or other static content. 
    Simply wrap the elements in a "fo:basic-links" if their parent has a @css:link attribute. 
    However, we cannot use "fo:basic-link" to wrap the "fo:footnote-call" because "fo:basic-link" is not a valid child of "fo:footnote"
    -->
    <xsl:template match="
        css:*[../@css:link][not(../@css:float='footnote')]
        |
        text()[../@css:link and string-length(normalize-space(.)) != 0]" priority="5">        
        <fo:basic-link>
            <xsl:call-template name="generate-link-attributes"/>
            <xsl:next-match/>
        </fo:basic-link>
    </xsl:template>


    <!-- 
        Generates the fo:basic-link attributes.
        
        Context: Any node that has a @link attribute.
    -->
    <xsl:template name="generate-link-attributes">
        <xsl:attribute name="show-destination" select="'new'"/>
        <xsl:attribute name="indicate-destination" select="'true'"/>

        <!-- Deals with special case in which the link is local, and starts with a hash. -->
        <xsl:variable name="link"
            select="
                if (starts-with(../@css:link, '#')) then
                    substring(../@css:link, 2)
                else
                    ../@css:link"/>

        <xsl:choose>
            <xsl:when test="key('ids', $link) or key('html-names', $link)">
                <!-- Is an internal link, there is an ID matching it in the document. -->
                <xsl:attribute name="internal-destination" select="$link"/>
            </xsl:when>
            <xsl:otherwise>
                <!-- Is an external link. -->
                <xsl:attribute name="external-destination" select="$link"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

</xsl:stylesheet>
