<?xml version="1.0" encoding="UTF-8"?>
<!-- 
    Deals with the overflow-wrap property. For break-word, it introduces soft 
    hyphens between the letters, but only for long words. The best implementation 
    would be at the FOP layout manager level.
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:oxy="http://www.oxygenxml.com/css2fo" xmlns:css="http://www.w3.org/1998/CSS"
    exclude-result-prefixes="xs oxy css" version="2.0">

    <!-- Do not pass it to the FO. -->
    <xsl:template match="@css:overflow-wrap" priority="2"/>

    <!-- 
      
      Matches the text that has an ancestor with a overflow-wrap property.
      
      
    -->
    <xsl:template match="text()[ancestor::*[@css:overflow-wrap][1]/@css:overflow-wrap = 'break-word']" priority="4">
        
      <xsl:analyze-string select="." regex="(\p{{L}}+)">
        <xsl:matching-substring>
          <xsl:choose>
            <xsl:when test="string-length(.) > 3">
              <!-- Matched a long word, add SHYs between its letters. -->
              <xsl:analyze-string select="." regex="(\p{{L}})(\p{{L}})?">
                <xsl:matching-substring>
                  <xsl:value-of select="regex-group(1)"/>
                  <xsl:if test="regex-group(2)">
                    <xsl:text>&#173;</xsl:text>
                    <xsl:value-of select="regex-group(2)"/>
                  </xsl:if>
                </xsl:matching-substring>
                <xsl:non-matching-substring>
                	<xsl:value-of select="." />
               	</xsl:non-matching-substring>
              </xsl:analyze-string>        
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="."/>
            </xsl:otherwise>         
          </xsl:choose>          
        </xsl:matching-substring>
        <xsl:non-matching-substring>
          <xsl:value-of select="." />
        </xsl:non-matching-substring>
      </xsl:analyze-string>
    </xsl:template>
  
</xsl:stylesheet>

